package il.ac.tau.cs.sw1.trie;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Set;

import org.junit.Before;
import org.junit.Test;

public class TrieTestCase {

	private Trie<String, String> names;
	private boolean addedNames;
	private TrieNode<String, String> node;

	@Before
	public void setUp() {
		names = new StringTrie<>();
		node = new StringTrieNode<String>();
		addedNames = false;
	}

	private void addNames() {
		if (!addedNames) {
			names.addKey("bill", "Murray");
			names.addKey("billie", "Holiday");
			names.addKey("bill", "Gates");
			names.addKey("bill", "Murray");
			names.addKey("albert", "Einstein");
			names.addKey("alan", "Turing");
			node.addSuffix("", "Jones");
			node.addSuffix("", "Jones");
			node.addSuffix("", "Smith");
		}
		addedNames = true;
	}

	@Test
	public void testInitialization() {
		assertEquals("Number of search results for \"bill\"", 0,
				names.searchByPrefix("bill").size());
		assertEquals("Number of search results for \"billy\"", 0,
				names.searchByPrefix("billy").size());
		assertEquals("Number of search results for \"billie\"", 0,
				names.searchByPrefix("billie").size());
		assertEquals("Number of search results for \"al\"", 0, names
				.searchByPrefix("al").size());
		assertEquals("Number of keys in new TrieNode", 0, node.getNumKeys());
		assertEquals("Number of values in new TrieNode", 0, node.getValues().size());
	}

	@Test
	public void testSearchBill() {
		addNames();
		Set<String> values = names.searchByPrefix("bill");
		assertEquals("Number of search results for \"bill\" after adding names", 3,
				values.size());
		assertTrue(
				"Search for \"bill\" after adding names should contain the last name Gates",
				values.contains("Gates"));
		assertTrue(
				"Search for \"bill\" after adding names should contain the last name Holiday",
				values.contains("Holiday"));
		assertTrue(
				"Search for \"bill\" after adding names should contain the last name Murray",
				values.contains("Murray"));
	}

	@Test
	public void testSearchBilly() {
		addNames();
		Set<String> values = names.searchByPrefix("billy");
		assertEquals("Number of search results for \"billy\" after adding names", 0,
				values.size());
	}
	
	@Test
	public void testSearchBillie() {
		addNames();
		Set<String> values = names.searchByPrefix("billie");
		assertEquals("Number of search results for \"billie\" after adding names", 1,
				values.size());
		assertTrue(
				"Search for \"billie\" after adding names should contain the last name Holiday",
				values.contains("Holiday"));
	}
	
	@Test
	public void testSearchAl() {
		addNames();
		Set<String> values = names.searchByPrefix("al");
		assertEquals("Number of search results for \"al\" after adding names", 2,
				values.size());
		assertTrue(
				"Search for \"al\" after adding names should contain the last name Holiday",
				values.contains("Einstein"));
		assertTrue(
				"Search for \"al\" after adding names should contain the last name Turing",
				values.contains("Turing"));
	}
	
	@Test
	public void testGetNumKeys() {
		addNames();
		assertEquals("Number of keys in TrieNode after adding suffixes", 3, node.getNumKeys());
	}
	
	@Test
	public void testGetValues() {
		addNames();
		Set<String> values = node.getValues();
		assertEquals("Number of values in new TrieNode after adding suffixes", 2, values.size());
		assertTrue(
				"TrieNode values after adding suffixes should contain the last name Jones",
				values.contains("Jones"));
		assertTrue(
				"TrieNode values after adding suffixes should contain the last name Smith",
				values.contains("Smith"));
	}
}
